// to make the proc_macros work inside the microrm crate; needed for tests and the metaschema.
extern crate self as microrm;

pub mod db;
mod query;
pub mod schema;

#[doc(hidden)]
pub use sqlite;

pub mod prelude {
    pub use crate::schema::entity::Entity;
    pub use crate::schema::{AssocMap, Database, IDMap};
    pub use microrm_macros::{Database, Entity};
}

// ----------------------------------------------------------------------
// Generically-useful database types
// ----------------------------------------------------------------------

#[derive(Debug)]
pub enum DBError {
    EmptyResult,
    UnknownValue(String),
    IncompatibleSchema,
    LogicError(&'static str),
    Sqlite(sqlite::Error),
    Query(String, sqlite::Error),
    JSON(serde_json::Error),
    LockError(String),
}

impl DBError {
    /// Converts a ::Sqlite error into a ::Query error by providing context
    pub(crate) fn sqlite_to_query(self, ctx: &str) -> Self {
        match self {
            Self::Sqlite(e) => Self::Query(ctx.to_string(), e),
            _ => self,
        }
    }
}

impl From<sqlite::Error> for DBError {
    fn from(value: sqlite::Error) -> Self {
        Self::Sqlite(value)
    }
}

impl<T> From<std::sync::PoisonError<T>> for DBError {
    fn from(value: std::sync::PoisonError<T>) -> Self {
        Self::LockError(value.to_string())
    }
}

pub type DBResult<T> = Result<T, DBError>;
